#!/bin/bash

# $Id:  $
# Copyright (c) 201e NetApp, Inc.
# All rights reserved.

PKGNAME=vpupgrader

MAINT_USER=maint
MAINT_GROUP=maintenance
MAINT_HOME="/home/${MAINT_USER}"
MAINT_SCRIPTS="${MAINT_HOME}/scripts"
LOG_DIR="/var/log"

upgradelog=$1
cmdoutlog="/var/log/upgrade-cmdout"
restart=false

cstat=0
function eval_it()
{
    eval "$@" &> ${cmdoutlog}
    cstat="$?"
    cat ${cmdoutlog} | tee -a ${upgradelog}
}

function echo_log()
{
    tstamp=`date +'%b %d %T'`
    echo "$@"
    echo "${tstamp}:  $@" >> ${upgradelog}
}

function log()
{
    tstamp=`date +'%b %d %T'`
    echo "${tstamp}:  $@" &>> ${upgradelog}
}

function vercomp(){
    local IFS=.
    local i first=($1) second=($2)
    for ((i=${#first[@]}; i<${#second[@]}; i++))
    do
        first[i]=0
        
    done
    for ((i=0; i<${#first[@]}; i++))
    do
        if [[ -z ${second[i]} ]]
        then
            second[i]=0
            
        fi
        if ((10#${first[i]} > 10#${second[i]}))
        then
            verresult="1"
            return 0
        fi
        if ((10#${first[i]} < 10#${second[i]}))
        then
            verresult="2"
            return 0
        fi
    done
    return 0
}

function UpdateKernel() {
    # Get the current installed version
    installedKernelVersion=$(uname -r | awk -F'-amd64' '{print $1}')
    log "installedKernelVersion=$installedKernelVersion"
    echo_log "installedKernelVersion=$installedKernelVersion"
    installedKernalVersionForCompare="${installedKernelVersion/-/.}"
    log "installedKernalVersionForCompare=$installedKernalVersionForCompare"
    echo_log "installedKernalVersionForCompare=$installedKernalVersionForCompare"

    isoKernelVersion=$(grep "linux_kernel_version" /APPLIANCE_BUILD | cut -d= -f2-)
    log "isoKernelVersion=${isoKernelVersion}"
    echo_log "isoKernelVersion=${isoKernelVersion}"
    isoKernelVersionForCompare="$(echo ${isoKernelVersion} | sed "s/-/./")"
    log "isoKernelVersionForCompare=${isoKernelVersionForCompare}"
    echo_log "isoKernelVersionForCompare=${isoKernelVersionForCompare}"

    verresult=0
    vercomp "${installedKernalVersionForCompare}" "${isoKernelVersionForCompare}"
    echo "vercompresult=$verresult"
    echo_log "vercompresult=$verresult"
    if [[ ${verresult} -eq 2 ]]; then
        log "New kernel available. Will upgrade Kernel and reboot the vApp"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-image-${isoKernelVersion}-amd64  &>> ${upgradelog}
	echo_log "After install linux-image.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-headers-${isoKernelVersion}-amd64 &>> ${upgradelog}
	echo_log "After install linux-headers.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-headers-${isoKernelVersion}-common  &>> ${upgradelog}
	echo_log "After install linux-headers-common.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-kbuild-4.9 &>> ${upgradelog}
	echo_log "After install linux-kbuild.... $?"
	
	echo_log "Setting dhcp-client-identifier to hardware to retain the same ip after kernel upgrade - this is required after upgrading to buster"
	sed -i 's/^#*\(send dhcp-client-identifier\).*$/\1 = hardware;/' /etc/dhcp/dhclient.conf

        # The grub.cfg is based on the OVA fresh install... we need to change some files because of some hardcoding
        filenames=`ls /etc/grub.d/*`
        for filename in $filenames; do
            sed -i 's/\/boot\/vmlinuz.*amd64/\/vmlinuz/g' $filename
            sed -i 's/\/boot\/initrd.img.*amd64/\/initrd.img/g' $filename
        done

	# The following change is required for changing the boot screen titile after upgrading from SRA 4.0 to VSC 7.2.1
	sed -i 's/Booting SRA Server 4.0 appliance...Please wait.../Booting VSC, VASA Provider, and SRA virtual appliance...Please wait.../g' /etc/grub.d/50_platform
	sed -i 's/Booting SRA Server 4.0P1 appliance...Please wait.../Booting VSC, VASA Provider, and SRA virtual appliance...Please wait.../g' /etc/grub.d/50_platform
	sed -i 's/Storage Replication Adapter Server 4.0 for ONTAP/VSC, VASA Provider, and SRA virtual appliance/g' /boot/grub/themes/netapp/theme.txt
	sed -i 's/Storage Replication Adapter Server 4.0P1 for ONTAP/VSC, VASA Provider, and SRA virtual appliance/g' /boot/grub/themes/netapp/theme.txt
	
    # Force the old eth0 naming for now in order to keep ova and iso upgrade consistent.
   	sed -i '/^linux \/vmlinuz/s/$/ net.ifnames=0/' /etc/grub.d/50_platform
	update-grub2 &>> ${upgradelog}

        restart="true"
    fi
}


#     ************* MAIN ***************
function main() {
    chattr -i /etc/passwd /etc/shadow
    echo_log "Upgrading virtual appliance OS and applications..."
    echo 'Acquire::cdrom::AutoDetect "false";' > /etc/apt/apt.conf
    echo_log 'Dpkg::Options { "--force-confold"; "--force-confdef"; }' >> /etc/apt/apt.conf

    echo "Before Starting update....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -qq update  --allow-unauthenticated &>> ${upgradelog}'
    echo "After Starting update.....$?" >> ${upgradelog}
    if [[ ${cstat} != 0 ]]
    then
        exit 1
    fi

    echo "Before Starting dist-upgrade....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated install openssh-server &>> ${upgradelog}'
    echo_log "After openssh-server...$?"

	echo "Installing iptables-persistent package before Starting dist-upgrade....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y -f --allow-unauthenticated install iptables-persistent &>> ${upgradelog}'
    echo_log "After iptables-persistent...$?"
    
    echo "Installing netfilter-persistent package before Starting dist-upgrade....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y -f --allow-unauthenticated install netfilter-persistent &>> ${upgradelog}'
    echo_log "After netfilter-persistent...$?"
	
	openVmToolsInstalled=0
	dpkg -l | grep open-vm-tools
	openVmToolsInstalled=$?
	if [[ ${openVmToolsInstalled} != 0 ]] 
	then
		echo_log "Starting installation of open-vm-tools....."
		eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated install open-vm-tools &>> ${upgradelog}'
		echo_log "Finished installation of open-vm-tools...$?"
	fi

	echo_log "Performing upgrade... Please wait..."
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get upgrade -y --force-yes --allow-unauthenticated &>> ${upgradelog}'
    echo_log "After upgrade... $?"
    if [[ ${cstat} != 0 ]]
    then
      echo_log "upgrade script failed"
      exit 1
    fi

	echo_log "Checking for re-encryption of bin files"
	eval_it '/home/maint/scripts/replace-hardcodedkeys-with-autogenerate.pl &>> ${upgradelog}'
	echo_log "Done with key updates"

    chattr -i /etc/passwd /etc/shadow
    echo_log "Performing dist-upgrade... Please wait... This might take several minutes..."
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get dist-upgrade -y -f --force-yes --allow-unauthenticated &>> ${upgradelog} '
    echo_log "After dist-upgrade..... $?" 

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt --fix-broken install -y --force-yes --allow-unauthenticated  &>> ${upgradelog}'
    echo_log "After fix-broken... $?"

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --allow-remove-essential --allow-unauthenticated install initramfs-tools &>> ${upgradelog}'
    echo_log "After initramfs-tools...$?"

    eval_it 'DEBIAN_FRONTEND=noninteractive apt --fix-broken install -y --force-yes --allow-unauthenticated  &>> ${upgradelog}'
    echo_log "After fix-broken... $?"
    if [[ ${cstat} != 0 ]]
    then
        exit 1
    fi

    apt-get -y purge cpp-4.6

	#Changes done for buster upgrade - remove "noapic" from boot parameters as it's causing kernel panic when booted with 2CPUs.
	echo_log "Removing boot parameter... noapic"
	sed -i 's/\<noapic\>//g' /etc/grub.d/50_platform
	update-grub2 &>> ${upgradelog}
	
    echo_log "After autoremove....$?"
    # Run any initialization/firstboot scripts that were added as part of the upgrade.
    /usr/sbin/service vplatform start 

    # Update appliance upgrade log
    # and version file.
    if [ ! -f /APPLIANCE_UPGRADE ] ; then
        cat <<EOF > /APPLIANCE_UPGRADE
###
### Original Install
###

EOF
        cat /APPLIANCE_BUILD >> /APPLIANCE_UPGRADE
    fi
    chattr -i /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    chmod 0666 /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    cat <<EOF >> /APPLIANCE_UPGRADE
###
### Upgrade $(date +'%b %d %T')
###

EOF

    cat /upgrade/APPLIANCE_BUILD >> /APPLIANCE_UPGRADE
    cp /upgrade/APPLIANCE_BUILD /APPLIANCE_BUILD
    chmod 0444 /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    chattr +i /APPLIANCE_BUILD /APPLIANCE_UPGRADE

	vmwareToolsInstalled=0
	dpkg -l | grep vmware-tools
	vmwareToolsInstalled=$?
	if [[ ${vmwareToolsInstalled} == 0 ]] 
	then
		echo_log "Uninstalling vmware-tools as it is present"
		apt-get -y purge vmware-tools
		echo_log "Uninstalled vmware-tools"
	fi
	
    UpdateKernel
    
           if grep -q 'inet static' "/etc/network/interfaces"; 
			then
  			echo "inet" > '/var/lib/vplatform/upgrade-network-CONFIG'
  			elif grep -q 'inet6 static' "/etc/network/interfaces";
			then
  			echo "inet6" > '/var/lib/vplatform/upgrade-network-CONFIG'
			else 
			echo "dhcp" > '/var/lib/vplatform/upgrade-network-CONFIG'
  			fi
            /home/maint/scripts/firstboot-network-config.pl --upgrade

            restart="true"
    
    # Restart the system if changes necessitate
    # These steps might not be required for now, but we may need them in future.
    if [ "${restart}" == "true" ]; then
        echo
        echo_log "Changes during virtual appliance upgrade necessitates a"
        echo_log "restart of this Virtual Machine..."
        /sbin/shutdown -r now
    fi

    exit 0
}

main "$@"
